using System;
using System.Drawing;
using System.Drawing.Printing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using System.IO;
using SubSystems.PDG;            // PDF Report Generator
using SubSystems.RC;             // RTF Code Generator namespace

namespace Demo
{
	/// <summary>
 /// Summary description for demo.
	/// </summary>
 public class Demo : System.Windows.Forms.Form
	{
     private System.Windows.Forms.GroupBox groupBox1;
     private System.Windows.Forms.Button OK;
     private System.Windows.Forms.Label Input1Title;
     private System.Windows.Forms.Label Input2Title;
     private System.Windows.Forms.TextBox Input1Text;
     private System.Windows.Forms.TextBox Input2Text;
     private System.Windows.Forms.ListBox Box;
     private System.Windows.Forms.Label label1;
     private System.Windows.Forms.TextBox Rtf;
     private System.Windows.Forms.Label label2;
     private System.Windows.Forms.CheckBox AutoRefresh;
     private System.Windows.Forms.LinkLabel Purchase;
 	private System.Windows.Forms.Label label3;
 	private System.Windows.Forms.TextBox OutputFile;
 	private System.Windows.Forms.CheckBox UseBuffer;
 	private System.Windows.Forms.Button GeneratePdf;
 	private System.Windows.Forms.GroupBox FileBox;
 	private System.Windows.Forms.Label Status;
		/// <summary>
		/// Required designer variable.
		/// </summary>
		private System.ComponentModel.Container components = null;

  ///////////////////////////////////////////////////////////////////////////////////////////////////
  ///    Constructor
  ///////////////////////////////////////////////////////////////////////////////////////////////////
  public Demo()
		{
    //
    // Required for Windows Form Designer support
    //
    InitializeComponent();

		}

  ///////////////////////////////////////////////////////////////////////////////////////////////////
  ///    Internal constants
  ///////////////////////////////////////////////////////////////////////////////////////////////////
  internal const int ID_NEW             =0;       // new docuemnt
  internal const int ID_INIT_SECT       =1;       // initialize section
  internal const int ID_BEGIN_TEXT      =2;       // begin text
  internal const int ID_RESET_CHAR_PROP =3;       // reset the character properties
  internal const int ID_SET_FONT        =4;       // set font/color
  internal const int ID_CHAR_STYLE_ID   =5;       // specify the character style id
  internal const int ID_RESET_PARA_PROP =6;       // reset the paragraph properties
  internal const int ID_PARA_CENTER     =7;       // centered paragraph
  internal const int ID_PARA_RIGHT      =8;       // right justified paragraph
  internal const int ID_PARA_DOUBLE_SPACE =9;     // double space paragraph
  internal const int ID_PARA_BORDER     =10;      // paragraph border
  internal const int ID_PARA_INDENT     =11;      // paragraph indentation
  internal const int ID_PARA_SPACE      =12;      // paragraph spacing
  internal const int ID_PARA_SHADE      =13;      // paragraph shading
  internal const int ID_PARA_BULLET     =14;      // paragraph bullet
  internal const int ID_PARA_NUMBERING  =15;      // paragraph numbering
  internal const int ID_PARA_TAB_STOP   =16;      // paragraph tab stops
  internal const int ID_PARA_STYLE_ID   =17;      // specify the paragraph style id
  internal const int ID_INSERT_TEXT     =18;      // append text
  internal const int ID_INSERT_TABLE    =19;      // insert a table
  internal const int ID_INSERT_PICT     =20;      // insert a picture file as linked
  internal const int ID_INSERT_LINK_PICT =21;     // insert a picture file as linked
  internal const int ID_PASTE_PICT      =22;      // paste from clipboard or memory
  internal const int ID_PAPER           =23;      // set paper size
  internal const int ID_MARGIN          =24;      // set margin
  internal const int ID_SECT_INFO       =25;      // section information
  internal const int ID_BEGIN_HDR       =26;      // begin the header group
  internal const int ID_END_HDR         =27;      // end the header group
  internal const int ID_BEGIN_FTR       =28;      // begin the footer group
  internal const int ID_END_FTR         =29;      // end the footer group
  internal const int ID_BEGIN_STYLESHEET =30;     // begin stylesheet
  internal const int ID_BEGIN_STYLEITEM =31;      // begin a style item
  internal const int ID_END_STYLEITEM   =32;      // end styleitem
  internal const int ID_END_STYLESHEET  =33;      // end stylesheet
  internal const int ID_BEGIN_FRAME     =34;      // begin a frame
  internal const int ID_END_FRAME       =35;      // end the current frame
  internal const int ID_BEGIN_TEXT_BOX  =36;      // begin a text box
  internal const int ID_END_TEXT_BOX    =37;      // end the current text box
  internal const int ID_DRAW_RECT       =38;      // draw a rectangle
  internal const int ID_DRAW_LINE       =39;      // draw a line
  internal const int ID_PAGE_BREAK      =40;      // insert a page break
  internal const int ID_SECT_BREAK      =41;      // insert a section break
  internal const int ID_INSERT_PAGE_NO  =42;      // insert a page number and total pages
  internal const int ID_INSERT_MERGE_FIELD =43;   // insert merge field
  internal const int ID_INSERT_TOC      =44;   // insert merge field
  internal const int ID_INSERT_RAW      =45;      // insert raw rtf code
  internal const int ID_INSERT_FNOTE    =46;      // insert footenote
  internal const int ID_OTHER           =47;      // testing code

  ///////////////////////////////////////////////////////////////////////////////////////////////////
  ///    Internal variables
  ///////////////////////////////////////////////////////////////////////////////////////////////////
  internal Pdg pdg=null;                         // pdf generator object
  internal Rcn doc=null;                         // document being created
  internal int CurFct=0;                         // current function id
  internal int StyleItemCount=0;                 // number of styles in the stylesheet
  internal int InputBoxHeight;                   // height of the input box

		/// <summary>
		/// Clean up any resources being used.
		/// </summary>
		protected override void Dispose( bool disposing )
		{
			if( disposing )
			{
				if(components != null)
				{
					components.Dispose();
				}
			}
			base.Dispose( disposing );
		}

		#region Windows Form Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			this.Input1Title = new System.Windows.Forms.Label();
			this.Input2Title = new System.Windows.Forms.Label();
			this.Input1Text = new System.Windows.Forms.TextBox();
			this.Input2Text = new System.Windows.Forms.TextBox();
			this.groupBox1 = new System.Windows.Forms.GroupBox();
			this.OK = new System.Windows.Forms.Button();
			this.Box = new System.Windows.Forms.ListBox();
			this.label1 = new System.Windows.Forms.Label();
			this.Rtf = new System.Windows.Forms.TextBox();
			this.label2 = new System.Windows.Forms.Label();
			this.AutoRefresh = new System.Windows.Forms.CheckBox();
			this.Purchase = new System.Windows.Forms.LinkLabel();
			this.UseBuffer = new System.Windows.Forms.CheckBox();
			this.OutputFile = new System.Windows.Forms.TextBox();
			this.label3 = new System.Windows.Forms.Label();
			this.GeneratePdf = new System.Windows.Forms.Button();
			this.FileBox = new System.Windows.Forms.GroupBox();
			this.Status = new System.Windows.Forms.Label();
			this.groupBox1.SuspendLayout();
			this.FileBox.SuspendLayout();
			this.SuspendLayout();
			// 
			// Input1Title
			// 
			this.Input1Title.Location = new System.Drawing.Point(8, 16);
			this.Input1Title.Name = "Input1Title";
			this.Input1Title.Size = new System.Drawing.Size(480, 16);
			this.Input1Title.TabIndex = 0;
			this.Input1Title.Text = "Input title 1";
			// 
			// Input2Title
			// 
			this.Input2Title.Location = new System.Drawing.Point(8, 64);
			this.Input2Title.Name = "Input2Title";
			this.Input2Title.Size = new System.Drawing.Size(480, 16);
			this.Input2Title.TabIndex = 1;
			this.Input2Title.Text = "Input title 2";
			// 
			// Input1Text
			// 
			this.Input1Text.Location = new System.Drawing.Point(8, 32);
			this.Input1Text.Name = "Input1Text";
			this.Input1Text.Size = new System.Drawing.Size(480, 20);
			this.Input1Text.TabIndex = 2;
			this.Input1Text.Text = "";
			// 
			// Input2Text
			// 
			this.Input2Text.Location = new System.Drawing.Point(8, 80);
			this.Input2Text.Name = "Input2Text";
			this.Input2Text.Size = new System.Drawing.Size(480, 20);
			this.Input2Text.TabIndex = 3;
			this.Input2Text.Text = "";
			// 
			// groupBox1
			// 
			this.groupBox1.Controls.Add(this.Input1Title);
			this.groupBox1.Controls.Add(this.Input1Text);
			this.groupBox1.Controls.Add(this.Input2Title);
			this.groupBox1.Controls.Add(this.Input2Text);
			this.groupBox1.Controls.Add(this.OK);
			this.groupBox1.Location = new System.Drawing.Point(224, 432);
			this.groupBox1.Name = "groupBox1";
			this.groupBox1.Size = new System.Drawing.Size(504, 144);
			this.groupBox1.TabIndex = 8;
			this.groupBox1.TabStop = false;
			// 
			// OK
			// 
			this.OK.Location = new System.Drawing.Point(416, 112);
			this.OK.Name = "OK";
			this.OK.TabIndex = 9;
			this.OK.Text = "OK";
			this.OK.Click += new System.EventHandler(this.OK_Click);
			// 
			// Box
			// 
			this.Box.Location = new System.Drawing.Point(8, 24);
			this.Box.Name = "Box";
			this.Box.Size = new System.Drawing.Size(200, 680);
			this.Box.TabIndex = 9;
			this.Box.SelectedIndexChanged += new System.EventHandler(this.Box_SelectedIndexChanged);
			// 
			// label1
			// 
			this.label1.Location = new System.Drawing.Point(8, 8);
			this.label1.Name = "label1";
			this.label1.Size = new System.Drawing.Size(160, 16);
			this.label1.TabIndex = 10;
			this.label1.Text = "Select a Function Call";
			// 
			// Rtf
			// 
			this.Rtf.Location = new System.Drawing.Point(224, 24);
			this.Rtf.Multiline = true;
			this.Rtf.Name = "Rtf";
			this.Rtf.Size = new System.Drawing.Size(504, 400);
			this.Rtf.TabIndex = 11;
			this.Rtf.Text = "textBox1";
			// 
			// label2
			// 
			this.label2.Location = new System.Drawing.Point(224, 8);
			this.label2.Name = "label2";
			this.label2.Size = new System.Drawing.Size(144, 16);
			this.label2.TabIndex = 12;
			this.label2.Text = "Intermediate RTF Code";
			// 
			// AutoRefresh
			// 
			this.AutoRefresh.Location = new System.Drawing.Point(632, 8);
			this.AutoRefresh.Name = "AutoRefresh";
			this.AutoRefresh.Size = new System.Drawing.Size(96, 16);
			this.AutoRefresh.TabIndex = 13;
			this.AutoRefresh.Text = "Auto Refresh";
			// 
			// Purchase
			// 
			this.Purchase.Font = new System.Drawing.Font("Microsoft Sans Serif", 8.25F, System.Drawing.FontStyle.Italic, System.Drawing.GraphicsUnit.Point, ((System.Byte)(0)));
			this.Purchase.LinkArea = new System.Windows.Forms.LinkArea(0, 0);
			this.Purchase.Location = new System.Drawing.Point(224, 688);
			this.Purchase.Name = "Purchase";
			this.Purchase.Size = new System.Drawing.Size(96, 16);
			this.Purchase.TabIndex = 14;
			this.Purchase.Text = "Purchase...";
			this.Purchase.LinkClicked += new System.Windows.Forms.LinkLabelLinkClickedEventHandler(this.Purchase_LinkClicked);
			// 
			// UseBuffer
			// 
			this.UseBuffer.Location = new System.Drawing.Point(16, 64);
			this.UseBuffer.Name = "UseBuffer";
			this.UseBuffer.Size = new System.Drawing.Size(216, 16);
			this.UseBuffer.TabIndex = 2;
			this.UseBuffer.Text = "Transfer file as buffer";
			// 
			// OutputFile
			// 
			this.OutputFile.Location = new System.Drawing.Point(16, 32);
			this.OutputFile.Name = "OutputFile";
			this.OutputFile.Size = new System.Drawing.Size(472, 20);
			this.OutputFile.TabIndex = 1;
			this.OutputFile.Text = "test.pdf";
			// 
			// label3
			// 
			this.label3.Location = new System.Drawing.Point(16, 16);
			this.label3.Name = "label3";
			this.label3.Size = new System.Drawing.Size(216, 16);
			this.label3.TabIndex = 0;
			this.label3.Text = "Output File Name (*.pdf)";
			// 
			// GeneratePdf
			// 
			this.GeneratePdf.Location = new System.Drawing.Point(528, 680);
			this.GeneratePdf.Name = "GeneratePdf";
			this.GeneratePdf.Size = new System.Drawing.Size(200, 24);
			this.GeneratePdf.TabIndex = 16;
			this.GeneratePdf.Text = "Generate PDF";
			this.GeneratePdf.Click += new System.EventHandler(this.GeneratePdf_Click);
			// 
			// FileBox
			// 
			this.FileBox.Controls.Add(this.Status);
			this.FileBox.Controls.Add(this.label3);
			this.FileBox.Controls.Add(this.OutputFile);
			this.FileBox.Controls.Add(this.UseBuffer);
			this.FileBox.Location = new System.Drawing.Point(224, 584);
			this.FileBox.Name = "FileBox";
			this.FileBox.Size = new System.Drawing.Size(504, 88);
			this.FileBox.TabIndex = 17;
			this.FileBox.TabStop = false;
			// 
			// Status
			// 
			this.Status.Location = new System.Drawing.Point(408, 64);
			this.Status.Name = "Status";
			this.Status.Size = new System.Drawing.Size(80, 16);
			this.Status.TabIndex = 3;
			// 
			// Demo
			// 
			this.AutoScaleBaseSize = new System.Drawing.Size(5, 13);
			this.ClientSize = new System.Drawing.Size(736, 709);
			this.Controls.Add(this.FileBox);
			this.Controls.Add(this.GeneratePdf);
			this.Controls.Add(this.Purchase);
			this.Controls.Add(this.AutoRefresh);
			this.Controls.Add(this.label2);
			this.Controls.Add(this.Rtf);
			this.Controls.Add(this.label1);
			this.Controls.Add(this.Box);
			this.Controls.Add(this.groupBox1);
			this.Name = "Demo";
			this.StartPosition = System.Windows.Forms.FormStartPosition.Manual;
			this.Text = "PDF Report Generator Demo";
			this.Load += new System.EventHandler(this.Demo_Load);
			this.groupBox1.ResumeLayout(false);
			this.FileBox.ResumeLayout(false);
			this.ResumeLayout(false);

		}
		#endregion
 
  /// <summary>
  /// The main entry point for the application.
  /// </summary>
  [STAThread]
  static void Main() 
  {
   Application.Run(new Demo());
  }

  protected override void OnPaintBackground(System.Windows.Forms.PaintEventArgs ev)
  {
    base.OnPaintBackground(ev);
  }
  
     private void Demo_Load(object sender, System.EventArgs e) {
         // fill in the list box
         SetFunctionName(ID_NEW,"New Document");
         SetFunctionName(ID_INIT_SECT,"Initialize Section");
         SetFunctionName(ID_BEGIN_TEXT,"Begin Section Text");
         SetFunctionName(ID_RESET_CHAR_PROP,"Reset Character Properties");
         SetFunctionName(ID_SET_FONT,"Set Font...");
         SetFunctionName(ID_CHAR_STYLE_ID,"Set Character Style Id");
         SetFunctionName(ID_RESET_PARA_PROP,"Reset Paragraph Properties");
         SetFunctionName(ID_PARA_CENTER,"Paragraph Centering");
         SetFunctionName(ID_PARA_RIGHT,"Paragraph Right Justification");
         SetFunctionName(ID_PARA_DOUBLE_SPACE,"Paragraph Double Spacing");
         SetFunctionName(ID_PARA_BORDER,"Paragraph Border");
         SetFunctionName(ID_PARA_INDENT,"Paragraph Indents");
         SetFunctionName(ID_PARA_SPACE,"Paragraph Spacing");
         SetFunctionName(ID_PARA_SHADE,"Paragraph Shading");
         SetFunctionName(ID_PARA_BULLET,"Paragraph Bullet");
         SetFunctionName(ID_PARA_NUMBERING,"Paragraph Numbering");
         SetFunctionName(ID_PARA_TAB_STOP,"Paragraph Tab Stops");
         SetFunctionName(ID_PARA_STYLE_ID,"Set Paragraph Style Id");
         SetFunctionName(ID_INSERT_TEXT,"Insert Text...");
         SetFunctionName(ID_INSERT_TABLE,"Insert Table");
         SetFunctionName(ID_INSERT_PICT,"Insert a Picture file...");
         SetFunctionName(ID_INSERT_LINK_PICT,"Insert a Linked Picture file...");
         //SetFunctionName(ID_PASTE_PICT,"Paste Picture");
         SetFunctionName(ID_PAPER,"Set Paper Size");
         SetFunctionName(ID_MARGIN,"Set Page Margin");
         SetFunctionName(ID_SECT_INFO,"Set Section Information");
         SetFunctionName(ID_BEGIN_HDR,"Begin Header Group");
         SetFunctionName(ID_END_HDR,"End Header Group");
         SetFunctionName(ID_BEGIN_FTR,"Begin Footer Group");
         SetFunctionName(ID_END_FTR,"End Footer Group");
         SetFunctionName(ID_BEGIN_STYLESHEET,"Begin Stylesheet");
         SetFunctionName(ID_BEGIN_STYLEITEM,"Begin a Stylesheet Item");
         SetFunctionName(ID_END_STYLEITEM,"End a Stylesheet Item");
         SetFunctionName(ID_END_STYLESHEET,"End Stylesheet");
         SetFunctionName(ID_BEGIN_FRAME,"Begin Frame Group");
         SetFunctionName(ID_END_FRAME,"End Frame Group");
         SetFunctionName(ID_BEGIN_TEXT_BOX,"Begin a Text Box Group");
         SetFunctionName(ID_END_TEXT_BOX,"End Text Box Group");
         SetFunctionName(ID_DRAW_RECT,"Draw a Rectangle");
         SetFunctionName(ID_DRAW_LINE,"Draw a Line");
         SetFunctionName(ID_PAGE_BREAK,"Insert a page break");
         SetFunctionName(ID_SECT_BREAK,"Insert a section break");
         SetFunctionName(ID_INSERT_PAGE_NO,"Insert Page Number");
         SetFunctionName(ID_INSERT_MERGE_FIELD,"Insert a Mail/Merge Field");
         SetFunctionName(ID_INSERT_TOC,"Insert Table-of-Content");
         SetFunctionName(ID_INSERT_RAW,"Insert Raw RTF Code...");
         SetFunctionName(ID_INSERT_FNOTE,"Insert footnote...");
         SetFunctionName(ID_OTHER," ");
      
         // hide the input area
         Input1Title.Visible=false;
         Input2Title.Visible=false;
         Input1Text.Visible=false;
         Input2Text.Visible=false;
         OK.Visible=false;
         
         Purchase.Links.Add(0,11, "www.subsystems.com/pdgw.htm");
         AutoRefresh.Checked=true;
         InputBoxHeight=Input1Text.Height;

         // start a new document
         pdg= new Pdg();             // create new pdf generator object
         CallFunction(ID_NEW,false);    
     }

     private void OK_Click(object sender, System.EventArgs e) {
         CallFunction(CurFct,true);   // execute the current function
     }

     private void Purchase_LinkClicked(object sender, System.Windows.Forms.LinkLabelLinkClickedEventArgs ev) {
        // Determine which link was clicked within the LinkLabel.
        Purchase.Links[Purchase.Links.IndexOf(ev.Link)].Visited = true;
        // Display the appropriate link based on the value of the LinkData property of the Link object.
        System.Diagnostics.Process.Start(ev.Link.LinkData.ToString());
     }

     private void Box_SelectedIndexChanged(object sender, System.EventArgs ev) 
     {
        int idx=Box.SelectedIndex;
        if (idx>=0) {
           ClsBox item=(ClsBox)Box.SelectedItem;
           CallFunction(item.value,false);        // execute the selected method
        } 
     }


     /******************************************************************************
         CallFunction:
         Call the specified API function.
     ******************************************************************************/
     internal new bool CallFunction(int fct, bool OnOK)
     {
         string str;

         CurFct=fct;                            // current function

         switch (fct) {
             case ID_NEW:                       // new document
                doc=pdg.PdgNewDocObject();
                StyleItemCount=0;
                break;

             case ID_BEGIN_STYLESHEET:          // begin the stylesheet group
                doc.RcgBeginStyleSheet();break;

             case ID_BEGIN_STYLEITEM:           // begin a stylesheet item
                doc.RcgBeginStyleItem(true,StyleItemCount,"Normal");
                StyleItemCount++;
                break;

             case ID_END_STYLEITEM:              // end current stylesheet item
                doc.RcgEndStyleItem();break;

             case ID_END_STYLESHEET:             // end current stylesheet group
                doc.RcgEndStyleSheet();break;


             case ID_PAPER:                     // set paper size to 8.5 by 11 inches
                //RcgPaper((int)(8.5*1440),(int)(11*1440),false);   // letter size portrait
                doc.RcgPaper(16834,11909,true);   // A4 in landscape orientation
                break;

             case ID_MARGIN:                     // set 1 inche margin around
                doc.RcgMargin(1440,1440,1440,1440,-1,-1);
                break;

             case ID_INIT_SECT:                 // initialize the section
                doc.RcgInitSect();break;

             case ID_SECT_INFO:                 // set the initial page number at 30
                doc.RcgSectInfo(1,0,true,30);break;

             case ID_BEGIN_HDR:                 // begin the header group
                doc.RcgBeginHdrFtr(Rcn.RCG_HDR);break;

             case ID_END_HDR:                   // end the header group
                doc.RcgEndHdrFtr();break;

             case ID_BEGIN_FTR:                 // begin the footer group
                doc.RcgBeginHdrFtr(Rcn.RCG_FTR);break;

             case ID_END_FTR:                   // end the footer group
                doc.RcgEndHdrFtr();break;

             case ID_BEGIN_FRAME:               // begin the footer group
                doc.RcgBeginFrame(true,2000,2000,4000,4000);break;

             case ID_END_FRAME:                 // end the footer group
                doc.RcgEndFrame();break;

             case ID_BEGIN_TEXT_BOX:            // begin the text box group
                doc.RcgBeginTextBox(true,1500,1500,2000,2000,-1,Color.White,Color.Black,20,false);
                break;

             case ID_END_TEXT_BOX:              // end the text box  group
                doc.RcgEndTextBox();break;

             case ID_DRAW_RECT:                 // draw a rectangle
                doc.RcgDrawRect(true,3500,3500,2000,2000,-1,Color.Magenta,Color.Black,20,false);
                break;

             case ID_DRAW_LINE:                 // draw a line
                doc.RcgDrawLine(true,2500,2500,3500,3500,-1,Color.Red,60);
                break;

             case ID_PAGE_BREAK:                 // page break
                doc.RcgInsertBreak(Rcn.RCG_BREAK_PAGE);break;

             case ID_SECT_BREAK:                 // page break
                doc.RcgInsertBreak(Rcn.RCG_BREAK_SECT);break;

             case ID_BEGIN_TEXT:                // text begins
                doc.RcgBeginSectText();break;

             case ID_RESET_CHAR_PROP:           // reset the character properties
                doc.RcgResetTextProp();break;

             case ID_RESET_PARA_PROP:           // reset the paragraph properties
                doc.RcgResetParaProp();break;

             case ID_SET_FONT:
                SetFont();break;

             case ID_CHAR_STYLE_ID:             // specify the character style id
                doc.RcgCharStyleId(1);break;

             case ID_PARA_CENTER:               // specify the paragraph centering
                doc.RcgParaFlags(Rcn.RCG_CENTER);break;

             case ID_PARA_RIGHT:               // specify the paragraph right justification
                doc.RcgParaFlags(Rcn.RCG_RIGHT);break;

             case ID_PARA_DOUBLE_SPACE:        // specify the paragraph double spacing
                doc.RcgParaFlags(Rcn.RCG_DOUBLE_SPACE);break;

             case ID_PARA_BORDER:              // Apply single width paragraph border
                doc.RcgParaFlags(Rcn.RCG_PARA_BOX);break;

             case ID_PARA_INDENT:              // Apply paragraph indentation
                doc.RcgParaIndent(1440,1440,1440);break;

             case ID_PARA_SPACE:               // Apply paragraph spacing
                doc.RcgParaSpace(720,720,400);break;

             case ID_PARA_SHADE:               // Apply paragraph shading
                doc.RcgParaShadeColor(30,Color.Black);break;

             case ID_PARA_BULLET:               // Apply paragraph bullet
                doc.RcgParaIndent(720,720,-720);
                doc.RcgParaBullet(true,Rcn.BLT_ROUND,0,1,null,null,0);
                break;

             case ID_PARA_NUMBERING:           // Apply paragraph Numbering
                doc.RcgParaIndent(720,720,-720);
                doc.RcgParaBullet(false,Rcn.NBR_DEC,0,1,"(",")",0);
                break;

             case ID_PARA_TAB_STOP:            // create paragraph tab stops
                doc.RcgParaTabStop(0,1440,0);      // at one inch
                doc.RcgParaTabStop(0,3*1440,0);    // at 3 inch
                doc.RcgParaTabStop(Rcn.TAB_RIGHT,5*1440,Rcn.TAB_DOT);  // right tab with dot leader at 5 inches
               break;

             case ID_PARA_STYLE_ID:             // specify the paragraph style id
                doc.RcgParaStyleId(1);break;

             case ID_INSERT_TEXT:
                if (OnOK) {
                   doc.RcgInsertText(Input1Text.Text);      // insert the text into the rtf buffer
                   DisableInputArea();
                }
                else EnableInputArea(true,"Please enter the text here and click OK",true);
                break;

             case ID_INSERT_RAW:
                if (OnOK) {
                   str=Input1Text.Text;
                   doc.RcgInsertRaw(str);      // insert the text into the rtf buffer
                   DisableInputArea();
                }
                else EnableInputArea(true,"Please enter raw RTF code here and click OK",true);
                break;

             case ID_INSERT_FNOTE:
                if (OnOK) {
                   str=Input1Text.Text;
                   doc.RcgInsertFootnote(null,str);      // insert the text into the rtf buffer
                   DisableInputArea();
                }
                else EnableInputArea(true,"Please enter footnote text here and click OK",true);
                break;

             case ID_INSERT_TABLE:
                InsertTable();
                break;

             case ID_INSERT_PICT:
                str="*.bmp";
                if (0==GetFileName(true,ref str,"bmp","Bitmap File(*.BMP)|*.BMP|Windows Metafile(*.WMF)|*.WMF|Jpeg File(*.JPG)|*.JPG|Png File(*.PNG)|*.PNG|")) break;
                doc.RcgInsertPictFile(str,4000,4000,false);      // save the code the file
                break;

             case ID_INSERT_LINK_PICT:
                str="*.bmp";
                if (0==GetFileName(true,ref str,"bmp","Bitmap File(*.BMP)|*.BMP|Windows Metafile(*.WMF)|*.WMF|Jpeg File(*.JPG)|*.JPG|Png File(*.PNG)|*.PNG|")) break;
                doc.RcgInsertPictFile(str,0,0,true);      // save the code the file
                break;

             //case ID_PASTE_PICT:
             //   doc.RcgPastePicture(0,null,0,0);
             //   break;

             case ID_INSERT_PAGE_NO:
                doc.RcgInsertText("Page# ");      // page number label
                doc.RcgInsertPageField(true);     // insert page number field
                doc.RcgInsertText(" Total Pages: ");
                doc.RcgInsertPageField(false);    // insert page count field
                break;

             case ID_INSERT_MERGE_FIELD:
                doc.RcgInsertMergeField("FieldName","FieldData");
                break;

             case ID_INSERT_TOC:
                doc.RcgInsertTocField(1,3);
                break;


             //case ID_SAVE_FILE:
             //   str="test.rtf";
             //   if (0==GetFileName(false,ref str,"rtf","RTF Format(*.RTF)|*.RTF|")) break;
             //   doc.RcgSaveFile(str);                 // save the code the file
             //   break;

             case ID_OTHER:                        // this code is for miscellaneous testing
                DemoOther();
                break;


         }

         if (AutoRefresh.Checked && doc!=null) ShowRtfCode();   // refresh the code text box

         return true;
     }

     //////////////////////////////////////////////////////////////////////////////////////////////////////////////////
     //////  Helper functions
     /////////////////////////////////////////////////////////////////////////////////////////////////////////////////
    /******************************************************************************
        DemoOther:
        Test code.
    ******************************************************************************/
    internal new void DemoOther()
    {
        InsertLocalLink();
    }
     
    /******************************************************************************
        InsertHyperlink:
        Insert a hyper link
    ******************************************************************************/
    internal new void InsertHyperlink()
    {
       doc.RcgInsertHyperlink("Sub Systems, Inc.",false,"http://www.subsystems.com");
    } 

    /******************************************************************************
        InsertLocalLink:
        Insert a local link to a bookmark in the report.  This bookmark needs to placed
        at a target location using the RcgInsertBookmark method
    ******************************************************************************/
    internal new void InsertLocalLink()
    {
       doc.RcgInsertHyperlink("Click to jump",true,"JumpHere");

       doc.RcgInsertText("\n");

       // add some random text before inserting the bookmark
       InsertTable();
       InsertTable();
       InsertTable();
       InsertTable();
       InsertTable();
       InsertTable();
         
       // place the bookmark
       doc.RcgInsertBookmark("JumpHere",true);          // begin bookmark
       
       doc.RcgTextColor(Color.Red,true);
       doc.RcgInsertText("Jump location\n");
       doc.RcgTextColor(Color.Black,true);
       
       doc.RcgInsertBookmark("JumpHere",false);         // end bookmark

       // add some random text after inserting the bookmark
       InsertTable();
       InsertTable();
       InsertTable();

    } 

    /******************************************************************************
        InsertTable:
        Insert a table. This function creates a table of 2 columns and 3 rows.
        The first row contains the header.  The table list the name and price of
        a couple products.
    ******************************************************************************/
    internal new bool InsertTable()
    {
        // Create the header row.
        //doc.RcgBeginTableRow(0,0,Rcn.RCG_CENTER,false,0);
        doc.RcgBeginTableRow2(0,0,Rcn.RCG_CENTER,false,200,200,200,200);
        
        doc.RcgNextCellInfo(5*1440,0,30,Color.White,20,40,20,40,0);
        doc.RcgNextCellInfo(1440,0,30,Color.White,40,20,20,40,0);

        // row 1 col 1
        doc.RcgBeginCellText();
        doc.RcgResetTextProp();
        doc.RcgTextFont("",0,Rcn.RCG_BOLD,true);
        doc.RcgInsertText("Products");
        doc.RcgEndCellText();

        // row 1 col 2
        doc.RcgBeginCellText();
        doc.RcgResetTextProp();
        doc.RcgTextFont("",0,Rcn.RCG_BOLD,true);
        doc.RcgInsertText("Price");
        doc.RcgEndCellText();

        doc.RcgEndTableRow();

        // create the second row
        doc.RcgBeginTableRow(0,0,Rcn.RCG_CENTER,false,0);
        doc.RcgNextCellInfo(5*1440,0,0,Color.White,20,40,40,20,0);
        doc.RcgNextCellInfo(1440,0,0,Color.White,40,20,40,20,0);

        // row 2 col 1
        doc.RcgBeginCellText();
        doc.RcgResetTextProp();
        doc.RcgInsertText("PDF Report Writer\r\n");
        
        doc.RcgInsertText("An example of a nested table");
        InsertNestedTable();    // an example of nesting tables

        doc.RcgEndCellText();

        // row 2 col 2
        doc.RcgBeginCellText();
        doc.RcgResetTextProp();
        doc.RcgInsertText("$439.00");
        doc.RcgEndCellText();

        doc.RcgEndTableRow();

        // create the third row
        doc.RcgBeginTableRow(0,0,Rcn.RCG_CENTER,false,0);
        doc.RcgNextCellInfo(5*1440,0,0,Color.White,20,40,20,20,0);
        doc.RcgNextCellInfo(1440,0,0,Color.White,40,20,20,20,0);

        // row 2 col 1
        doc.RcgBeginCellText();
        doc.RcgResetTextProp();
        doc.RcgInsertText("ReportEase Plus");
        doc.RcgEndCellText();

        // row 2 col 2
        doc.RcgBeginCellText();
        doc.RcgResetTextProp();
        doc.RcgInsertText("$459.00");
        doc.RcgEndCellText();

        doc.RcgEndTableRow();

        return true;
    }

    /******************************************************************************
        InsertNestedTable:
        Insert a table. This function creates a table of 2 columns and 3 rows.
        The first row contains the header.  The table list the name and price of
        a couple products.
    ******************************************************************************/
    internal new bool InsertNestedTable()
    {
        // Create the header row.
        doc.RcgBeginTableRow(0,0,Rcn.RCG_CENTER,false,0);
        doc.RcgNextCellInfo(3*1440,0,30,Color.White,20,40,20,40,0);
        doc.RcgNextCellInfo(1440,0,30,Color.White,40,20,20,40,0);

        // row 1 col 1
        doc.RcgBeginCellText();
        doc.RcgResetTextProp();
        doc.RcgTextFont("",0,Rcn.RCG_BOLD,true);
        doc.RcgInsertText("Products");
        doc.RcgEndCellText();

        // row 1 col 2
        doc.RcgBeginCellText();
        doc.RcgResetTextProp();
        doc.RcgTextFont("",0,Rcn.RCG_BOLD,true);
        doc.RcgInsertText("Price");
        doc.RcgEndCellText();

        doc.RcgEndTableRow();

        // create the second row
        doc.RcgBeginTableRow(0,0,Rcn.RCG_CENTER,false,0);
        doc.RcgNextCellInfo(3*1440,0,0,Color.White,20,40,40,20,0);
        doc.RcgNextCellInfo(1440,0,0,Color.White,40,20,40,20,0);

        // row 2 col 1
        doc.RcgBeginCellText();
        doc.RcgResetTextProp();
        doc.RcgInsertText("TE Developer's Kit\r\n");
        doc.RcgInsertText("Another level of table nesting:");
        InsertNestedTable2();
        doc.RcgInsertText("Back into the outer nested table:");
        doc.RcgEndCellText();

        // row 2 col 2
        doc.RcgBeginCellText();
        doc.RcgResetTextProp();
        doc.RcgInsertText("$439.00");
        doc.RcgEndCellText();

        doc.RcgEndTableRow();

        // create the third row
        doc.RcgBeginTableRow(0,0,Rcn.RCG_CENTER,false,0);
        doc.RcgNextCellInfo(3*1440,0,0,Color.White,20,40,20,20,0);
        doc.RcgNextCellInfo(1440,0,0,Color.White,40,20,20,20,0);

        // row 2 col 1
        doc.RcgBeginCellText();
        doc.RcgResetTextProp();
        doc.RcgInsertText("ReportEase Plus");
        doc.RcgEndCellText();

        // row 2 col 2
        doc.RcgBeginCellText();
        doc.RcgResetTextProp();
        doc.RcgInsertText("$459.00");
        doc.RcgEndCellText();

        doc.RcgEndTableRow();

        return true;
    }

    /******************************************************************************
        InsertNestedTable2:
        Insert a table. This function creates a table of 2 columns and 3 rows.
        The first row contains the header.  The table list the name and price of
        a couple products.
    ******************************************************************************/
    internal new bool InsertNestedTable2()
    {
        // Create the header row.
        doc.RcgBeginTableRow(0,0,Rcn.RCG_CENTER,false,0);
        doc.RcgNextCellInfo(2400,0,30,Color.White,20,40,20,40,0);
        doc.RcgNextCellInfo(1440,0,30,Color.White,40,20,20,40,0);

        // row 1 col 1
        doc.RcgBeginCellText();
        doc.RcgResetTextProp();
        doc.RcgTextFont("",0,Rcn.RCG_BOLD,true);
        doc.RcgInsertText("Products");
        doc.RcgEndCellText();

        // row 1 col 2
        doc.RcgBeginCellText();
        doc.RcgResetTextProp();
        doc.RcgTextFont("",0,Rcn.RCG_BOLD,true);
        doc.RcgInsertText("Price");
        doc.RcgEndCellText();

        doc.RcgEndTableRow();

        // create the second row
        doc.RcgBeginTableRow(0,0,Rcn.RCG_CENTER,false,0);
        doc.RcgNextCellInfo(2400,0,0,Color.White,20,40,40,20,0);
        doc.RcgNextCellInfo(1440,0,0,Color.White,40,20,40,20,0);

        // row 2 col 1
        doc.RcgBeginCellText();
        doc.RcgResetTextProp();
        doc.RcgInsertText("TE Developer's Kit");
        doc.RcgEndCellText();

        // row 2 col 2
        doc.RcgBeginCellText();
        doc.RcgResetTextProp();
        doc.RcgInsertText("$439.00");
        doc.RcgEndCellText();

        doc.RcgEndTableRow();

        // create the third row
        doc.RcgBeginTableRow(0,0,Rcn.RCG_CENTER,false,0);
        doc.RcgNextCellInfo(2400,0,0,Color.White,20,40,20,20,0);
        doc.RcgNextCellInfo(1440,0,0,Color.White,40,20,20,20,0);

        // row 2 col 1
        doc.RcgBeginCellText();
        doc.RcgResetTextProp();
        doc.RcgInsertText("ReportEase Plus");
        doc.RcgEndCellText();

        // row 2 col 2
        doc.RcgBeginCellText();
        doc.RcgResetTextProp();
        doc.RcgInsertText("$459.00");
        doc.RcgEndCellText();

        doc.RcgEndTableRow();

        return true;
    }

     /******************************************************************************
         DisableInputArea:
         Disable the input area.
     ******************************************************************************/
     internal new bool DisableInputArea()
     {
         Input1Title.Text="";
         Input2Title.Text="";
         Input1Text.Text="";
         Input2Text.Text="";

         // reset the input box sizes
         Input1Text.Height=InputBoxHeight;
         Input2Text.Height=InputBoxHeight;
         Input1Text.Multiline=false;

         Input1Title.Visible=false;
         Input2Title.Visible=false;
         Input1Text.Visible=false;
         Input2Text.Visible=false;
         OK.Visible=false;

         return true;
     }

     /******************************************************************************
         EnableInputArea:
         Enable the first input area.
     ******************************************************************************/
     internal new bool EnableInputArea(bool first, string title,bool enlarge)
     {
         TextBox tb=first?Input1Text:Input2Text;
         Label   lbl=first?Input1Title:Input2Title;

         if (enlarge) {
            tb.Height=3*InputBoxHeight;
            tb.Multiline=true;
         }

         lbl.Text=title;

         lbl.Visible=true;
         tb.Visible=true;
         OK.Visible=true;

         tb.Focus();

         return true;
     }

     /******************************************************************************
         SetFont:
         Show the common dialog to accept new font/color for the text
     ******************************************************************************/
     internal new bool SetFont()
     {
         FontDialog fd=new FontDialog();
         Font font;
         int style;
         int  TwipsSize;
         FontStyle fs=FontStyle.Regular;

         font=new Font("Arial",(float)12,fs);

         fd.Font=font;
         fd.Color=Color.Black;
         fd.FontMustExist=true;
         fd.ShowColor=true;

         if (fd.ShowDialog()==DialogResult.Cancel) return false;

         // Extract the font dialog values
         font=fd.Font;

         TwipsSize=(int)(font.SizeInPoints*20);   // convert PointSize to TwipsSize
         style=0;
         if (font.Bold)      style|=Rcn.RCG_BOLD;
         if (font.Italic)    style|=Rcn.RCG_ITALIC;
         if (font.Underline) style|=Rcn.RCG_ULINE;
         if (font.Strikeout) style|=Rcn.RCG_STRIKE;
      
         doc.RcgTextFont(font.Name,(int)font.SizeInPoints,style,true);

         doc.RcgTextColor(fd.Color,true);

         font.Dispose();

         return true;
     }

     
     /******************************************************************************
         ShowRtfCode:
         Show the rtf code so far generated.
     ******************************************************************************/
     internal new bool ShowRtfCode()
     {
         string pCode=doc.RcgGetData();

         Rtf.Text=pCode;   // show the generated code
         Rtf.Refresh();

         return true;
     }
     internal new void SetFunctionName(int id, string name)
     {
        Box.Items.Add(new ClsBox(name,id));
     } 
     
     string ReadFromFile(string file) 
     {
        string str=null;

        StreamReader InFile=null;
        if ((InFile=new StreamReader(file))!=null) {
          str=InFile.ReadToEnd();
          InFile.Close();
        }

        return str;
     } 
 
     bool WriteToFile(string file, string OutData) 
     {
        bool result=false;

        StreamWriter OutFile=null;
        try {
          if ((OutFile=new StreamWriter(file))!=null) {
            OutFile.Write(OutData);
            OutFile.Close();
            result=true;
          }
        }
        catch (Exception) {
        } 


        return result;
     } 

     /******************************************************************************
         GetFileName:
         This function gets the file name from the user.  The first argument is
         true for 'open' dialog box, and false for the 'Save As' dialog box.
         The second argument can be used to specify the initial file name and
         get the file selection from the user.  The third argument (filter) must
         be '|' for delimters for wild card pairs.
     ******************************************************************************/
     internal new int GetFileName(bool open,ref string file,string DefExt, string filter)
     {
         int result=0;

         int len=filter.Length;
         if (len>0 && filter[len-1]=='|') filter=filter.Substring(0,len-1);  // strip the last '|' character
       
         if (open) {
            OpenFileDialog fd = new OpenFileDialog();
            fd.DefaultExt=DefExt;
            fd.FileName=file;
            fd.Filter=filter;
            fd.FilterIndex=(filter.Length>0)?1:0;
            fd.RestoreDirectory=true;
            fd.ShowHelp=true;
            if (fd.ShowDialog()==DialogResult.OK) {
               file=fd.FileName;
               result=fd.FilterIndex;   // base 1
            }
         }
         else {
            SaveFileDialog fd = new SaveFileDialog();
            fd.DefaultExt=DefExt;
            fd.FileName=file;
            fd.Filter=filter;
            fd.FilterIndex=(filter.Length>0)?1:0;
            fd.RestoreDirectory=true;
            fd.ShowHelp=true;
         
            if (fd.ShowDialog()==DialogResult.OK) {
               file=fd.FileName;
               result=fd.FilterIndex;   // base 1
            }
         }

         return result;
     }


     /////////////////////////////////////////////////////////////////////////////////////////////////////////////
     //    Testing/Debugging code                                                                    
     /////////////////////////////////////////////////////////////////////////////////////////////////////////////
     /***************************************************************************
         ToInt:
         Convert string to integer
     ****************************************************************************/
     internal new int ToInt(TextBox item)
     {
        return ToInt(item.Text);
     }

     internal new int ToInt(string txt)
     {
        try {
           return (int)Convert.ToInt32(txt);
        } 
        catch (System.Exception)
        {
           return 0;
        }
     } 

  internal new bool True(int val)
  {
     return (val!=0);
  }

 	private void GeneratePdf_Click(object sender, System.EventArgs e)
	 {
     bool result=false;
     string OutFile=OutputFile.Text;

     if (UseBuffer.Checked) {   // first generate the pdf in a string and then write it to the file
        string PdfStr=pdg.PdgGenerateReportString(doc);
        if (PdfStr!=null) result=pdg.PdgWriteToFile(OutFile,PdfStr);
     } 
     else {
        result=pdg.PdgGenerateReportFile(doc,OutFile);
     } 

     Status.Text=result?"Done!":"Error!";
     if (result) System.Diagnostics.Process.Start(OutFile);  // display pdf

	 }


 }

 /////////////////////////////////////////////////////////////////////////////////////////////////////////////
 ///// object class to insert list box items
 /////////////////////////////////////////////////////////////////////////////////////////////////////////////
 internal class ClsBox {          // list box/combo-box object
   internal string item;         // string
   internal int    value;        // item value
   internal string TextValue;    // text value
   internal object ObjectValue;  // object value
   public ClsBox(string ArgItem,int ArgValue)
   {
      item=ArgItem;
      value=ArgValue;
   }
   public ClsBox(string ArgItem,string ArgValue)
   {
      item=ArgItem;
      TextValue=ArgValue;
   }
   public ClsBox(string ArgItem,object ArgValue)
   {
      item=ArgItem;
      ObjectValue=ArgValue;
   }
   public override string ToString() {return item;}
}
  
}
